import	unittest
from	pygwarts.tests.irma.unit_irma_contrib			import *
from	pygwarts.tests.irma.unit_irma_contrib_intercept	import *
from	pygwarts.tests.irma.unit_irma_access			import *
from	pygwarts.tests.irma.unit_irma_access_volume		import *
from	pygwarts.tests.irma.unit_irma_access_bookmark	import *
from	pygwarts.tests.irma.unit_irma_access_handler	import *
from	pygwarts.tests.irma.unit_irma_access_inducer	import *
from	pygwarts.tests.irma.unit_irma_access_annex		import *
from	pygwarts.tests.irma.unit_irma_access_utils		import *
from	pygwarts.tests.irma.unit_irma_shelf				import *
from	pygwarts.tests.irma.unit_irma_shelf_casing		import *








if	__name__ == "__main__" : unittest.main(verbosity=2)







