import	unittest
from	pygwarts.tests.magical.unit_magical_chest						import *
from	pygwarts.tests.magical.unit_magical_spells						import *
from	pygwarts.tests.magical.unit_magical_key_chest					import *
from	pygwarts.tests.magical.unit_magical_time_turner					import *
from	pygwarts.tests.magical.unit_magical_time_turner_utils			import *
from	pygwarts.tests.magical.unit_magical_time_turner_timers			import *
from	pygwarts.tests.magical.unit_magical_phstone						import *
from	pygwarts.tests.magical.unit_magical_transmutations				import *
from	pygwarts.tests.magical.unit_magical_controlled_transmutations	import *








if	__name__ == "__main__" : unittest.main(verbosity=2)







