#                                                                                                              
#                                                                                                              
#                                                                                                              
#                                                                                                              
#                                                      .                                                       
#                                                      }d,                                                     
#                                                      1a'                                                     
#                                                      (k.                                                     
#                                                      rm"                                                     
#                                                     'Xz<.                                                    
#                                                   "n+0Yn{[.                                                  
#                                                 .-Qdn]#+ahc^                                                 
#                                                 \Ckk]-#,QkkmI                                                
#                                               ,xLhZ> )*`'jkkd}.                                              
#                                              ]cOhn'  cd; .~wkhz".                                            
#                                            ^jcbq+   'Uv-.  `uhkq!                                            
#                                           ,XzhQ,    ubqQ>    {kkq;                                           
#                                          ,nLhj'.    >uUx`    .>mkd[                                          
#                                         ]uOm~,~}/tt/!ra/)/tt(-!^Uhhc".                                       
#                                       `uUxvrr/t//nXU~Ch)vYzxfrrrxcuvO_.                                      
#                                     .")/Lu]j0dX/?l` .kd{..^!]fUbX)\Lut<.                                     
#                                    .:rU)(CQ)!.      "oL(       '_uqzjuJ].                                    
#                                  .`rC]xbc<         '/OUv>.        .?Jdjx0~                                   
#                                 .iZ((dUl           :O0JC(.           ~0q\Ox'.                                
#                                 lp]uqf.            .>tLu.             'zbtZu                                 
#                               ",Zj\dn.              :zbn.              'Jk/h[l.                              
#                              _C1b!hq'               ^mkr                ;abuq?z^                             
#                            `xvnzYjku                ;ak|                 Lhv*!ow~                            
#                          .:C/bjXYxkn                }a0r.                CkCo!akh(.                          
#                          ?XXax`td)hO.              iCQLJj`              `kkZk`~wkhz^                         
#                        `xXwq_. "bfwh1              -hOJCn:            ..nkkkf..`vhkq+.                       
#                      .~UChz"    _hxdh{.            .(uOZ`.           ..rhkkJ'    ?pkhx'                      
#                     .(Lqh|.      -hcphn'            [UkL             "JkkkJ'      :Zkku.                     
#                     (Obw+.       .iQOZkm)`          _ZkU          ."/dkkbt'        `nhhc^                    
#                   `nqhc^.         ..]Qqdkdj!..      {kkz       .^{Xbkkbx:.          ._dkq+.                  
#                  <mkd]                ixwkkkpX/]>". xwOt:..,<}fUbkkkU{`               >pkhx'                 
#                'fkdZcffffffffffffffffffffczCbkkkkhj/pqqZC?qhkkkh0czxfffffffffffffffffffnQmbZ>                
#              ."OoaoaaaaaaaaaaaaaaaaaaaaaaaopXccvc0>cqZUZv]nYuczcZoaaaaaaaaaaaaaaaaaaaaaaaoaao{.              
#                                                 .  :LYc|1.  .                                                
#                                                    .*bpc[                                                    
#                                                    'hkkd|                                                    
#                                                     ~hkY .                                                   
#                                                     l/(-.    
#
# Ain't fan!