import	unittest
from	pygwarts.tests.filch.unit_filch_apppuf			import *
from	pygwarts.tests.filch.unit_filch_transportclaw	import *
from	pygwarts.tests.filch.unit_filch_nettherin		import *
from	pygwarts.tests.filch.unit_filch_linkindor		import *
from	pygwarts.tests.filch.unit_filch_marauders_map	import *








if	__name__ == "__main__" : unittest.main(verbosity=2)







