import	unittest
from	pygwarts.tests.hagrid.unit_hagrid_init		import *
from	pygwarts.tests.hagrid.unit_hagrid_order		import *
from	pygwarts.tests.hagrid.unit_hagrid_sifting	import *
from	pygwarts.tests.hagrid.unit_hagrid_peels		import *
from	pygwarts.tests.hagrid.unit_hagrid_BlindPeek	import *
from	pygwarts.tests.hagrid.unit_hagrid_DraftPeek	import *
from	pygwarts.tests.hagrid.unit_hagrid_flourish	import *
from	pygwarts.tests.hagrid.unit_hagrid_easy1		import *
from	pygwarts.tests.hagrid.unit_hagrid_easy2		import *
from	pygwarts.tests.hagrid.unit_hagrid_easy3		import *
from	pygwarts.tests.hagrid.unit_hagrid_easy4		import *
from	pygwarts.tests.hagrid.unit_hagrid_easy5		import *
from	pygwarts.tests.hagrid.unit_hagrid_easy6		import *
from	pygwarts.tests.hagrid.unit_hagrid_easy7		import *
from	pygwarts.tests.hagrid.unit_hagrid_easy8		import *
from	pygwarts.tests.hagrid.unit_hagrid_easy9		import *
from	pygwarts.tests.hagrid.unit_hagrid_easy10	import *
from	pygwarts.tests.hagrid.unit_hagrid_easy11	import *
from	pygwarts.tests.hagrid.unit_hagrid_easy12	import *
from	pygwarts.tests.hagrid.unit_hagrid_easy13	import *
from	pygwarts.tests.hagrid.unit_hagrid_easy14	import *
from	pygwarts.tests.hagrid.unit_hagrid_easy15	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium1	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium2	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium3	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium4	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium5	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium6	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium7	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium8	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium9	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium10	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium11	import *
from	pygwarts.tests.hagrid.unit_hagrid_medium12	import *








if	__name__ == "__main__" : unittest.main(verbosity=2)







